﻿# pgnTcec2Scid converts a TCEC PGN from CuteChess to Scid vs PC format
$FileIn = "C:\Chess\Stockfish\Tcec\Superfinal14\Tcec14_Superfinal.pgn"
$streamOut = [System.IO.StreamWriter] "C:\Chess\Stockfish\Tcec\Superfinal14\Tcec14ScidSuperfinal.pgn"
$TemplateCfs = "C:\Users\Chess\Documents\Powershell\pgnTcec2ScidCfs.txt"
$nbrGame, $strMoves, $cr, $strOpening = 0, "", "`r`n", ""

# Remove-Comment("1. e4 {book, mb=+0+0+0+0+0,}") returns "1. e4". Remove the comment and the preceding whitespace
function Remove-Comment([string]$cmt)
{
    $indOpen = $cmt.IndexOf("{")
    if ($indOpen -gt -1)
    {
        if ($cmt[$indOpen-1] -eq " ")
        {
            $indOpen--
        }
        if ($indOpen -gt 0)
        {
            $LeftPart = $cmt.Substring(0, $indOpen)
        }
        else
        {
            $LeftPart = ""
        }
        $indClose = $cmt.IndexOf("}")
        if ($indClose -gt 0) 
        {
            if ($indClose+1 -lt $cmt.Length)
            {
                return $LeftPart + $cmt.Substring($indClose+1)  
            }
            else
            {
                return $LeftPart
            }
        }
    }
    return $cmt #return the initial comment
}
 
# Remove-mbComment("{book, mb=+0+0+0+0+0,}") returns "{book}"
function Remove-mbComment([string]$cmt)
{
    $indCmt = $cmt.IndexOf(", mb=")
    if ($indCmt -gt 1)
    {
        $lenMax = $cmt.Length
        for ($idx = $indCmt + 5; $idx -le $lenMax; $idx++)
        {
            $Char = $cmt[$idx]
            if ($Char -eq ",")
            {
                return $cmt.Substring(0, $indCmt) + $cmt.Substring($idx+1)
            }
        }
    }
    return $cmt #return the comment
}

# Add the eval score of a move in a PGN comment
function Comment-Score([string]$eval)
{
    if ($eval.Length -gt 0)
    {
        if ($eval.StartsWith("-"))
        {
            $SignedScore = $eval
        }
        else # Add the "+" symbol before the score
        {
            $SignedScore = "+" + $eval
        }
        return " {" + $SignedScore + "}"
    }
    else
    {
        return ""
    }
}

# Add move number before each White's move of the Principal Variation
function Num-PrincipalVariation([int]$NumMove, [boolean]$isBlack, [string]$strPv)
{
    $Result = $cr + "(" # Open parenthesis to open the principal variation
    if ($isBlack)
    {
        $Result = $Result + $NumMove.ToString() + "... "
        $indHalfMove = 1
    }
    else
    {
        $Result = $Result + $NumMove.ToString() + ". "
        $indHalfMove = 0
    }
    $indMove,$lenMax, $indChar, $NbrMove = $NumMove, $strPv.Length, 0, 0
    for ($indChar = 0; $indChar -le $lenMax; $indChar++)
    {
        $Char = $strPv[$indChar]
        if ($Char -eq " ")
        {
            $NbrMove ++
            $indHalfMove++
            $Result = $Result + " "
            if ($indHalfMove -gt 1)
            {
                $indHalfMove = 0
                $indMove++
                $Result = $Result + $indMove.ToString() + ". "
            }
        }
        else
        {
            $Result = $Result + $Char
        }
    }
    if ($NbrMove -gt 0)
    {
        $Result = $Result + ")" # Close the principal variation
    }
    else
    {
        $Result = ""
    }
    return $Result # Return the string of the principal variation
}

$LC, $SF = 0, 0
Get-Content $FileIn | ForEach-Object { # means For each line of text in the PGN where $_ represents the current line
    if($_ -match "^\[.*\]"){ # Reproduce pairs of [tag-name "tag_value"] without any changes
        $streamOut.WriteLine($_) # Optionnally, you can put a "#" before "Write-output" to disable the generation of the wished section as in the following:
    }
    elseif($_ -match "^{[^}]*}"){ # Skip first comment  of UCI prms starting at the first column.
        # RegExpr:    ^first col. "{" + all except "}" + "}" see: get-help about_regular_expressions
        #              "[^}]*"
        #Write-output ("Delete first comment : " + $_)
    }
    elseif($_ -match "\} "){ # During the opening book, both White and Black moves are on the same line
        ConvertFrom-String -InputObject $_ -Delimiter "\} " -OutVariable PgnOpening -PropertyNames WhiteMove, BlackMove | Out-Null
        $strOpening = $strOpening + (Remove-Comment($PgnOpening.WhiteMove + "} ")) + (Remove-Comment($PgnOpening.BlackMove)) + " "
    }
    elseif($_ -match "^0-1" -or $_ -match "^1-0" -or $_ -match "^1/2-1/2"){ # Game terminator?
        $strMoves | ConvertFrom-String -TemplateFile $TemplateCfs -UpdateTemplate -OutVariable PgnMoves| Out-Null
        $nbrGame++
        if ($_ -match "^0-1")
        {
            if ($nbrGame -bAnd 1) {$SF++} else {$LC++}
        }
        elseif ($_ -match "^1-0")
        {
            if ($nbrGame -bAnd 1) {$LC++} else {$SF++}
        }
        Write-output ("Game, " + ($nbrGame) + ", Chess moves found, " + ($PgnMoves.Count) + ", LC, " + $LC.ToString() + ", SF, " + $SF.ToString())
        $indMove = 0
        $PgnMoves | ForEach-Object {
            $strPV = Num-PrincipalVariation ($_.Nbr) (0) ($_.CommentW.pv)  # Convert Principal Variation (pv) into a Scid variation between parenthesis
            $streamOut.WriteLine($_.Nbr.ToString() + ". " + $_.White + (Comment-Score($_.CommentW.wv)) + $strPV)

            if ($_.CommentB -ne $null)
            {
                $strPV = Num-PrincipalVariation ($_.Nbr) (1) ($_.CommentB.pv)
                $streamOut.WriteLine($_.Black + (Comment-Score($_.CommentB.wv)) + $strPV)
            }
            $indMove++
        }
        $streamOut.WriteLine($_ + $cr) # Game terminator
        #if ($nbrGame -gt 6)
        #{
            #$streamOut.close()
            #break
        #}
        $strMoves = ""
    }
    else # Remaining moves in the middle game and the end game
    {
        if ($strOpening.Length -gt 0)
        {
            $streamOut.WriteLine($strOpening + "{opening}")
            $strOpening = ""
        }
        $strMoves = $strMoves + (Remove-mbComment($_)) + $cr
    }
}
$streamOut.close()
#expected result:
#Game, 1, Chess moves found, 127, LC, 0, SF, 0
#Game, 2, Chess moves found, 112, LC, 0, SF, 0
#Game, 3, Chess moves found, 63, LC, 0, SF, 0
#Game, 4, Chess moves found, 48, LC, 0, SF, 0
#Game, 5, Chess moves found, 59, LC, 0, SF, 0
#Game, 6, Chess moves found, 75, LC, 0, SF, 0
#Game, 7, Chess moves found, 48, LC, 0, SF, 1
#Game, 8, Chess moves found, 119, LC, 0, SF, 1
#...
#Game, 100, Chess moves found, 33, LC, 9, SF, 10
# end of pgnTcec2Scid.ps1